/*
 * line.cpp
 *
 *  Created on: Sep 3, 2016
 *      Author: zeil
 */

#include <limits>

#include <line.h>

using namespace std;


Line::Line (Point thruPoint1, Point thruPoint2)
  : p1(thruPoint1), p2(thruPoint2)
{}

double Line::y (int x) const
{
  if (p1.x == p2.x)
    return numeric_limits<double>::max();
  else
    return (double)p1.y + ((double)(p2.y - p1.y) / (p2.x - p1.x)) * (x - p1.x);
}

double Line::x (int y) const
{
  if (p1.y == p2.y)
    return numeric_limits<double>::max();
  else
    return  (double)p1.x + ( (double)(p2.x - p1.x) / (p2.y - p1.y)) * (y - p1.y);
}


bool Line::crossesSegment (const Line& segment)
{
  // Not really a general implementation - we'll only need this for
  // horizontal or vertical segments
  if (segment.p1.x == segment.p2.x)
    {
      double x = segment.p1.x;
      double y = this->y(x);
      return bounded (y, segment.p1.y, segment.p2.y);
    }
  else if (segment.p1.y == segment.p2.y)
    {
      double y = segment.p1.y;
      double x = this->x(y);
      return bounded(x, segment.p1.x, segment.p2.x);
    }
  else if (p1.x == p2.x)
    {
      double x = p1.x;
      double y = segment.y(x);
      return bounded(x, segment.p1.x, segment.p2.x)
	&& bounded(y, segment.p1.y, segment.p2.y);
    }
  else if (p1.y == p2.y)
    {
      double y = p1.y;
      double x = segment.x(y);
      return bounded(x, segment.p1.x, segment.p2.x)
	&& bounded(y, segment.p1.y, segment.p2.y);
    }
  else
    return false; // not implemented
}

bool Line::bounded (double v, double v1, double v2) const
{
  return (v != std::numeric_limits<double>::max()
	  && ((v >= v1 && v <= v2)
	      || (v >= v2 && v <= v1)));
}





